#include <HCSR04.h>

UltraSonicDistanceSensor distanceSensor(9, 8);  // Initialize sensor that uses digital pins 9 and 8.

//Variables globales
int compteur;                       //Compte le nombre de dixièmes de seconde
char compteur_delai_pulsation;      //Compte le nombre de tranches de 100 microsecondes


//Fonction d'initialisation
void setup() 
{
  Serial.begin(9600);                   //Initialisation du port série
  #define bouton_porte 11               //Bouton qui actionne fermeture/ouverture de la porte de garage
  pinMode(bouton_porte, OUTPUT);        //Bouton porte en sortie
  #define DEL 13                        //Broche de la DEL
  pinMode(DEL, OUTPUT);                 //Broche de DEL en sortie
  #define detect_mouv 10                //Détecteur de passage sous la porte
  pinMode(detect_mouv, INPUT_PULLUP);   //Détecteur en entrée
//  #define detect_porte 12                //Détecteur de porte ouverte
//  pinMode(detect_porte, INPUT_PULLUP);  //Détecteur en entrée
}

//Fonction de detection de porte ouverte, via le module d'ultrasons.
bool verif_pres_porte()
{
  float distance;
  distance = distanceSensor.measureDistanceCm();    // Lecture de la distance de la porte.
  if((distance > 30) & (distance < 60)) return true;    // Porte detectee lorsque la distance est entre 30 et 60 cm.
  else return false;
}
 
// Boucle Principale
void loop() 
{
  #define temps_limite 36000  // Temps limite avant de fermer porte (une heure)
  
  if(verif_pres_porte())    //Vérif si porte ouverte
  {  //Oui. Vérifier s'il y a pulsation venant du détecteur de passage, ce qui dit qu'il n'y a pas de passage.
    for(compteur_delai_pulsation = 0; compteur_delai_pulsation < 80; compteur_delai_pulsation++)
    {
      if (digitalRead(detect_mouv) == true) break; //Si pulsation détectée, sortir de la boucle
      delayMicroseconds(100);
    }
    if(compteur_delai_pulsation >= 80) // Y a t'il eu passage dans le détecteur?
    {
      compteur = 0; //Pas de pulsation détectée, donc passage détecté. On ré-initialise le compteur.
    }
    else        //Pulsation détectée car compteur_delai_pulsation < 80
    { // on continue à compter
      compteur = compteur + 1;              //Incrémenter le compteur
      digitalWrite(DEL, HIGH);              //Allume la DEL pour 50ms
      delay(50);                       
      digitalWrite(DEL, LOW);               //Éteins la DEL pour 50ms
      delay(50);
      if((compteur == temps_limite) & (verif_pres_porte()))   //Si compteur expiré ET porte ouverte
      {
        digitalWrite(bouton_porte, HIGH);   //Appuie sur bouton pour fermer pendant 200ms
        delay(200);                       
        digitalWrite(bouton_porte, LOW);    //Relache le bouton
        digitalWrite(DEL, HIGH);            //Allume la DEL pour 3 sec
        delay(3000);                       
        digitalWrite(DEL, LOW);             //Éteint la DEL      
        compteur = 0;                       //Réinitialise le compteur
      }
    }
  }
  else                                      //Si porte fermée
  {
    compteur = 0;                           //Réinitialise le compteur
  }    
}
 

